"""Auto-use fixtures for pytest.

WARNING: Importing this module will automatically apply these fixtures. If you want to selectively
enable fixtures in a different context, you can import directly from the `fixtures.general` module.


Usage:

```python
from airbyte_cdk.test.fixtures import auto
# OR
from airbyte_cdk.test.fixtures.auto import *
```
"""
